/*
   SPDX-FileCopyrightText: 2015-2025 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include <QObject>

class RichTextComposerImagesTest : public QObject
{
    Q_OBJECT
public:
    explicit RichTextComposerImagesTest(QObject *parent = nullptr);
    ~RichTextComposerImagesTest() override;
};
